/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.document.library.taglib.servlet.taglib;

import com.liferay.document.library.taglib.internal.display.context.DLViewFileVersionDisplayContextUtil;
import com.liferay.document.library.taglib.internal.servlet.ServletContextUtil;
import com.liferay.portal.kernel.repository.model.FileVersion;
import com.liferay.taglib.util.IncludeTag;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.PageContext;

/**
 * @author Alejandro Tardín
 */
public class MimeTypeStickerTag extends IncludeTag {

	@Override
	public int doStartTag() {
		return EVAL_BODY_INCLUDE;
	}

	public String getCssClass() {
		return _cssClass;
	}

	public FileVersion getFileVersion() {
		return _fileVersion;
	}

	public void setCssClass(String cssClass) {
		_cssClass = cssClass;
	}

	public void setFileVersion(FileVersion fileVersion) {
		_fileVersion = fileVersion;
	}

	@Override
	public void setPageContext(PageContext pageContext) {
		super.setPageContext(pageContext);

		setServletContext(ServletContextUtil.getServletContext());
	}

	@Override
	protected void cleanUp() {
		super.cleanUp();

		_cssClass = null;
		_fileVersion = null;
	}

	@Override
	protected String getPage() {
		return _PAGE;
	}

	@Override
	protected void setAttributes(HttpServletRequest httpServletRequest) {
		httpServletRequest.setAttribute(
			"liferay-document-library:mime-type-sticker:cssClass", _cssClass);
		httpServletRequest.setAttribute(
			"liferay-document-library:mime-type-sticker:" +
				"dlViewFileVersionDisplayContext",
			DLViewFileVersionDisplayContextUtil.
				getDLViewFileVersionDisplayContext(
					httpServletRequest,
					(HttpServletResponse)pageContext.getResponse(),
					_fileVersion));
	}

	private static final String _PAGE = "/mime_type_sticker/page.jsp";

	private String _cssClass;
	private FileVersion _fileVersion;

}